@@ -4,11 +4,9 @@ from typing import Optional
 
 import trio
 
-from .h2 import H2Server
-from .h11 import H11Server
 from .lifespan import Lifespan
-from .wsproto import WebsocketServer
-from ..asgi.run import H2CProtocolRequired, H2ProtocolAssumed, WebsocketProtocolRequired
+from .server import Server
+from .statsd import StatsdLogger
 from ..config import Config, Sockets
 from ..typing import ASGIFramework
 from ..utils import (
@@ -16,41 +14,12 @@ from ..utils import (
     load_application,
     MustReloadException,
     observe_changes,
+    repr_socket_addr,
     restart,
     Shutdown,
 )
 
 
-async def serve_stream(app: ASGIFramework, config: Config, stream: trio.abc.Stream) -> None:
-    if config.ssl_enabled:
-        try:
-            with trio.fail_after(config.ssl_handshake_timeout):
-                await stream.do_handshake()
-        except (trio.BrokenResourceError, trio.TooSlowError):
-            return  # Handshake failed
-        selected_protocol = stream.selected_alpn_protocol()
-    else:
-        selected_protocol = "http/1.1"
-
-    if selected_protocol == "h2":
-        protocol = H2Server(app, config, stream)
-    else:
-        protocol = H11Server(app, config, stream)  # type: ignore
-    try:
-        await protocol.handle_connection()
-    except WebsocketProtocolRequired as error:
-        protocol = WebsocketServer(  # type: ignore
-            app, config, stream, upgrade_request=error.request
-        )
-        await protocol.handle_connection()
-    except H2CProtocolRequired as error:
-        protocol = H2Server(app, config, stream, upgrade_request=error.request)
-        await protocol.handle_connection()
-    except H2ProtocolAssumed as error:
-        protocol = H2Server(app, config, stream, received_data=error.data)
-        await protocol.handle_connection()
-
-
 async def worker_serve(
     app: ASGIFramework,
     config: Config,
@@ -59,6 +28,8 @@ async def worker_serve(
     shutdown_event: Optional[EventType] = None,
     task_status: trio._core._run._TaskStatus = trio.TASK_STATUS_IGNORED,
 ) -> None:
+    config.set_statsd_logger_class(StatsdLogger)
+
     lifespan = Lifespan(app, config)
     reload_ = False
 
@@ -82,22 +53,28 @@ async def worker_serve(
                         sock.listen(config.backlog)
 
                 ssl_context = config.create_ssl_context()
-                listeners = [
-                    trio.ssl.SSLListener(
-                        trio.SocketListener(trio.socket.from_stdlib_socket(sock)),
-                        ssl_context,
-                        https_compatible=True,
+                listeners = []
+                binds = []
+                for sock in sockets.secure_sockets:
+                    listeners.append(
+                        trio.SSLListener(
+                            trio.SocketListener(trio.socket.from_stdlib_socket(sock)),
+                            ssl_context,
+                            https_compatible=True,
+                        )
                     )
-                    for sock in sockets.secure_sockets
-                ]
-                listeners.extend(
-                    [
-                        trio.SocketListener(trio.socket.from_stdlib_socket(sock))
-                        for sock in sockets.insecure_sockets
-                    ]
-                )
-                task_status.started()
-                await trio.serve_listeners(partial(serve_stream, app, config), listeners)
+                    bind = repr_socket_addr(sock.family, sock.getsockname())
+                    binds.append(f"https://{bind}")
+                    await config.log.info(f"Running on {bind} over https (CTRL + C to quit)")
+
+                for sock in sockets.insecure_sockets:
+                    listeners.append(trio.SocketListener(trio.socket.from_stdlib_socket(sock)))
+                    bind = repr_socket_addr(sock.family, sock.getsockname())
+                    binds.append(f"http://{bind}")
+                    await config.log.info(f"Running on {bind} over http (CTRL + C to quit)")
+
+                task_status.started(binds)
+                await trio.serve_listeners(partial(Server, app, config), listeners)
 
         except MustReloadException:
             reload_ = True
